<?php
/**
 * A registry for several data types and objects. Objects should be added by
 * addInstance() and therefore must implement the interface Registerable.
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007 - 2009 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class Registry extends BaseFrameworkSystem implements Register {
	/**
	 * Instance of this class
	 */
	private static $registryInstance = null;

	/**
	 * Wether the registry is initialized
	 */
	private static $initialized = false;

	/**
	 * Instance registry
	 */
	private $instanceRegistry = array();

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Singleton getter for self instance. This class has no factory pattern
	 * because here is no need for special parameters.
	 *
	 * @return	$registryInstance	Instance of this class
	 */
	public final static function getRegistry () {
		// Is an instance there?
		if (is_null(self::$registryInstance)) {
			// Not yet, so create one
			self::$registryInstance = new Registry();
		} // END - if

		// Return the instance
		return self::$registryInstance;
	}

	/**
	 * Checks or sets wether the registry has been initialized. This had only
	 * be done once.
	 *
	 * @param	$initialized	Wether the registry is initialized
	 * @return	$initialized	Wether the registry is initialized
	 */
	public final static function isInitialized ($initialized = null) {
		// If no parameter (null by default) is provided we want to get the
		// parameter. If set to e.g. "OK" then the registry is set to
		// "initialized".
		if (is_null($initialized)) {
			// Get status if initialized
			return self::$initialized;
		} else {
			// Registry is initialized!
			self::$initialized = true;
		}
	}

	/**
	 * Checks wether an instance key was found
	 *
	 * @param	$instanceKey	The key holding an instance in registry
	 * @return	$exists			Wether the key exists in registry
	 */
	public function instanceExists ($instanceKey) {
		// Does this key exists?
		$exists = (isset($this->instanceRegistry[$instanceKey]));

		// Return the result
		return $exists;
	}

	/**
	 * Adds/overwrites a new instance to the registry at the given key
	 *
	 * @param	$instanceKey		The key to identify the instance
	 * @param	$objectInstance		An instance we shall store
	 * @return	void
	 */
	public function addInstance ($instanceKey, Registerable $objectInstance) {
		$this->instanceRegistry[$instanceKey] = $objectInstance;
	}

	/**
	 * Gets a registered instance or null if not found
	 *
	 * @param	$instanceKey		The key to identify the instance
	 * @return	$objectInstance		An instance we shall store
	 */
	public function getInstance ($instanceKey) {
		// By default the instance is not in registry
		$objectInstance = null;

		// Is the instance there?
		if ($this->instanceExists($instanceKey)) {
			$objectInstance = $this->instanceRegistry[$instanceKey];
		} // END - if

		// Return the result
		return $objectInstance;
	}
}

// [EOF]
?>
